#!/usr/bin/python3

import argparse
import re
import signal
import subprocess
import sys
import threading


def main():
    p = argparse.ArgumentParser(
        description='Run a program, and kill it if it is silent for too long.'
                    'After the arguments, specify the program to run prefixed '
                    'with -- .')
    p.add_argument('--timeout', type=parse_timeout,
                   help='Timeout (in seconds, if no unit specified')

    our_args = sys.argv[1:]
    cmd = []
    if '--' in our_args:
        index = our_args.index('--')
        cmd = our_args[index + 1:]
        our_args = our_args[:index]

    args = p.parse_args(our_args)
    if not cmd:
        p.error('No command specified')

    run_cmd(cmd, args.timeout)


def parse_timeout(arg):
    m = re.match(r'(\d+) *(s(ec(ond)?s?)?|m(in(ute)?s?)?|h((ou)?rs?)?)?', arg)
    if not m:
        raise argparse.ArgumentTypeError('Un-parseable time')
    value = int(m.group(1))
    unit = m.group(2)
    if unit:
        unit = unit[0]
        if unit == 'm':
            return value * 60
        if unit == 'h':
            return value * 60 * 60
    return value


def run_cmd(cmd, timeout):
    p = subprocess.Popen(cmd, encoding='UTF-8', bufsize=1,
                         stdout=subprocess.PIPE)
    watchdog = Watchdog(timeout, p)
    watchdog.start()
    while True:
        line = p.stdout.readline()
        watchdog.event.set()
        sys.stdout.write(line)
        sys.stdout.flush()
        if p.poll() != None:
            sys.exit(p.returncode)


class Watchdog(threading.Thread):
    def __init__(self, timeout, process):
        super(Watchdog, self).__init__()
        self.daemon = True
        self.timeout = timeout
        self.process = process
        self.event = threading.Event()

    def run(self):
        while True:
            ev = self.event.wait(self.timeout)
            if ev is True:
                self.event.clear()
            else:
                sys.stderr.write(
                    '\nTimed out after {} seconds of inactivity...\n'
                    .format(self.timeout))
                sys.stderr.flush()
                self.process.send_signal(signal.SIGINT)
                return


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from cffi import FFI


ffibuilder = FFI()
ffibuilder.set_source('_fib',
    """
    int fib(int n) {
       if (n < 0) {
           return -1; // Raise
       }
       if (n < 2) {
           return 1;
       }
       return fib(n - 1) + fib(n - 2);
    }
    """,
    libraries=[])
ffibuilder.cdef("""
    int fib(int n);
""")


if __name__ == '__main__':
    ffibuilder.compile(verbose=True)



#!/usr/bin/python3

# Some headers are generated at build time, so we ship them in pypy3 (anch-dep).
# Others are static and shipped in pypy3-dev (arch-indep).
# Because we are cherry-picking them from different places, verify that all the
# expected headers have been shipped.

import sys
from glob import glob
from os import listdir
from os.path import basename


def installed_headers(binpkg):
    path = f'debian/{binpkg}.install'
    with open(path, encoding='utf-8') as f:
        for line in f:
            if line.startswith('#'):
                continue
            parts = line.split()
            if len(parts) == 2:
                pattern, dest = parts
            elif len(parts) == 3 and parts[1] == '=>':
                pattern, _, dest = parts
            else:
                print(f'Failure to parse {path}: {line}')
                sys.exit(1)
            if dest.rstrip('/') == '/usr/include/pypy3.11':
                for fn in glob(pattern):
                    yield basename(fn)


installed = (
    set(installed_headers('pypy3-dev'))
    | set(installed_headers('pypy3')))
built_headers = set(
    basename(fn) for fn in listdir('include/pypy3.11')
    if fn.endswith('.h'))

if installed != built_headers:
    if built_headers - installed:
        print("Missing headers:", built_headers - installed)
    if installed - built_headers:
        print("Extra headers:", installed - built_headers)
    sys.exit(1)



#!/usr/bin/python3

import os
import re
import subprocess


def cffi_version():
    with open('lib_pypy/cffi/__init__.py', encoding='UTF-8') as f:
        for line in f:
            m = re.match(r'^__version__ = "([0-9.]+)(?:\.dev\d+)?"$', line)
            if m:
                return m.group(1)
    raise Exception('Version not found')


def target_version():
    with open('lib_pypy/cffi/recompiler.py', encoding='UTF-8') as f:
        for line in f:
            m = re.match(r'^VERSION_CHAR16CHAR32 = (0x[0-9A-F]+)$', line)
            if m:
                return int(m.group(1), 16)
    raise Exception('Version not found')


def backend_supported_versions():
    versions = {}
    with open('pypy/module/_cffi_backend/cffi1_module.py',
              encoding='UTF-8') as f:
        for line in f:
            m = re.match(r'^VERSION_(MIN|MAX) *= (0x[0-9A-F]+)$',
                         line)
            if m:
                versions[m.group(1)] = int(m.group(2), 16)
                if len(versions) == 2:
                    return versions['MIN'], versions['MAX']
    raise Exception('Versions not found')


def pypy_abi():
    if 'pypy3' not in subprocess.check_output(('dh_listpackages',)).split():
        return ''
    soabi = subprocess.check_output((
        'pypy/goal/pypy3.11-c', '-c',
        'import sysconfig; print(sysconfig.get_config_var("SOABI"))'))
    return soabi.strip().replace('-', '-abi-')


cffi = cffi_version()
versions = backend_supported_versions()
target = target_version()

subst = {
    'cffi': cffi,
    'min': versions[0],
    'max': versions[1],
    'target': target,
}
with open('debian/pypy3-lib.substvars', 'a', encoding='UTF-8') as f:
    f.write('cffi:Provides=pypy3-cffi (= {cffi}), '
            'pypy3-cffi-backend-api-min (= {min}), '
            'pypy3-cffi-backend-api-max (= {max}), '
            'pypy3-cffi-backend-api-{target}\n'.format(**subst))

with open('debian/pypy3.substvars', 'a', encoding='UTF-8') as f:
    f.write('pypy3-abi={}\n'.format(pypy_abi()))

path = 'debian/pypy3-lib/usr/share/python3/dist'
os.makedirs(path)
with open(os.path.join(path, 'pypy3-cffi'), 'w', encoding='UTF-8') as f:
    f.write('cffi pypy3-cffi-backend-api-min (<= {target}), '
            'pypy3-cffi-backend-api-max (>= {target})\n'
            .format(**subst))
